#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <unistd.h>

#include <string.h>
#include <sys/types.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "arec.h"

static void insert_ana(arec *arec, ana_req *ana_r);
static void insert(arec *arec, allocation *alloc);
static void insert_asa(arec *arec, addr_range *ar);
static void insert_own(arec *arec, allocation *alloc);
static void remove_alloc(arec *arec, allocation *alloc);
static list *overlap(arec *arec, addr_range *ar);
static int in_use(arec *arec, addr_range *ar);
static addr_range *sel_addr(arec *arec, int num, unsigned int end_time);
static list *allocations(arec *arec);
static int is_own(arec *arec, addr_range *ar);
static int arec_allocation_callback(void *obj, unsigned int n);
static int arec_asa_callback(void *obj, unsigned int n);
/*static void set_asa_exp_timer(arec *arec, int time);*/
/*static int asa_exp_timer_callback(void *obj, unsigned int n);*/

/*------------------------------------------------------------------*/
/* These structs are used to pass state to callback functions */
typedef struct alloc_arec_ alloc_arec;

struct alloc_arec_ {

    arec *arec;
    allocation *alloc;

};

typedef struct ar_arec_ ar_arec;

struct ar_arec_ {

    arec *arec;
    addr_range *ar;

};

/*------------------------------------------------------------------*/
/* syncronizes the disk cache */
static void sync_cache(arec *a) {
  
    FILE *f;
    int i;
    char *err;
    
    f = fopen(a->cfg->arec.cache_fname, "w");
    if(f == NULL){
        sprintf(err,"Opening file %s for writing" , a->cfg->arec.cache_fname);
        perror(err);
        exit(1);
    }
    fprintf(f, "allocation-record \"%s\" {\n", a->cfg->name);

    for(i = 0; i < a->asa_ranges->length(a->asa_ranges); i++) {
        addr_range *ar;
        struct in_addr first_addr, last_addr;
        char *first, *last, *etime_date; 
        time_t etime;

    
        ar = (addr_range *)(a->asa_ranges->elem_at(a->asa_ranges, i));

        first_addr.s_addr = ar->first_addr(ar);
        last_addr.s_addr  = ar->last_addr(ar);
  
        first = strdup(inet_ntoa(first_addr));
        last = strdup(inet_ntoa(last_addr));
        etime = ar->end_time(ar);
        etime_date = ctime(&etime);

        fprintf(f, "\tasa(%s, %s, %lu);\n\t // Expires at %s\n", first, last, etime, etime_date);
        free(first);
        free(last);

	    
    }

    for(i = 0; i < a->cur_allocs->length(a->cur_allocs); i++) {   
        allocation *alloc;
        struct in_addr first_addr, last_addr, owner_addr;
        char *first, *last, *etime_date, *owner; 
        time_t etime, stime;
        int prealloc;

        alloc = (allocation *)(a->cur_allocs->elem_at(a->cur_allocs, i));

        if(alloc->owner_addr(alloc) != ALLOCATION_OWN) {
            first_addr.s_addr = alloc->first_addr(alloc);
            last_addr.s_addr  = alloc->last_addr(alloc);
            owner_addr.s_addr  = alloc->owner_addr(alloc);
      
            first = strdup(inet_ntoa(first_addr));
            last = strdup(inet_ntoa(last_addr));
            owner = strdup(inet_ntoa(owner_addr));
            etime = alloc->end_time(alloc);
            etime_date = ctime(&etime);
            stime = alloc->start_time(alloc);
            prealloc = alloc->is_prealloc(alloc);
      
            fprintf(f, "\tallocation(%s, %s, %lu, %lu, %d, %s);\n\t // Expires at %s\n",
                    first, last, etime, stime, prealloc, owner, etime_date);
            /*      fprintf(stderr, "\tallocation(%s, %s, %lu, %lu, %d, %s);\n\t // Expires at %s\n",
                    first, last, etime, stime, prealloc, owner, etime_date);*/

            free(first);
            free(last);
            free(owner);


        }

	    
    }

    for(i = 0; i < a->own_allocs->length(a->own_allocs); i++) {
        allocation *alloc;
        struct in_addr first_addr, last_addr;
        char *first, *last, *etime_date; 
        time_t etime, stime;
        int prealloc;

        alloc = (allocation *)(a->own_allocs->elem_at(a->own_allocs, i));
        first_addr.s_addr = alloc->first_addr(alloc);
        last_addr.s_addr  = alloc->last_addr(alloc);
    
        first = strdup(inet_ntoa(first_addr));
        last = strdup(inet_ntoa(last_addr));
        etime = alloc->end_time(alloc);
        etime_date = ctime(&etime);
        stime = alloc->start_time(alloc);
        prealloc = alloc->is_prealloc(alloc);
    
        fprintf(f, "\town(%s, %s, %lu, %lu, %d);\n\t // Expires at %s\n",
                first, last, etime, stime, prealloc, etime_date);
        /*    fprintf(stderr, "\town(%s, %s, %lu, %lu, %d);\n\t // Expires at %s\n",
              first, last, etime, stime, prealloc, etime_date);*/
        free(first);
        free(last);

    
    }
    for(i = 0; i < a->ana_reqs->length(a->ana_reqs); i++){
        ana_req *ana_r = (ana_req *)a->ana_reqs->elem_at(a->ana_reqs, i);
        fprintf(f, "\tana(%lu, %lu, %lu, %lu);\n",
                ana_r->numaddr,
                ana_r->end_time,
                ana_r->clock_skew,
                ana_r->rseq);
    }
    fprintf(f, "};\n");

    fclose(f);
}

static void insert_ana(arec *arec, ana_req *ana_r){
    int i = 0;
    /* Find out if it is already in the list. */
    i = arec->ana_reqs->length(arec->ana_reqs) - 1;
    while(i >= 0){
        ana_req *areq = (ana_req *)arec->ana_reqs->elem_at(arec->ana_reqs, i);
        if((areq->numaddr == ana_r->numaddr) &&
           (areq->end_time == ana_r->end_time) &&
           (areq->rseq == ana_r->rseq)){
            /* Already in list, return. */
            return;
        }
        i--;
    }

    /* Not in list, so we add it. */
    arec->ana_reqs->insert(arec->ana_reqs, ana_r);
    sync_cache(arec);
}

/*------------------------------------------------------------------*/
/*
 * This function inserts an allocation into the allocation record.
 */
static void insert(arec *arec, allocation *alloc){

    int end_alloc_time = alloc->end_time(alloc);
    alloc_arec *aa = (alloc_arec *)malloc(sizeof(alloc_arec));


    /*  printf("arec: insert ");
        alloc->debug_print(alloc);*/
    /* if the allocation's end time is closer than the closest in the timer,
       substitute the time event for the old time event with the new */

    /* if there are no allocations in the record,
       simply insert the new allocation */

    if(arec->cur_allocs->is_empty(arec->cur_allocs)) {
   
        arec->next_end_time = end_alloc_time;
        arec->next_end_alloc = alloc;
        aa->alloc = alloc;
        aa->arec = arec;
        arec->next_end_tim_id = arec->tim->set(arec->tim,
                                               arec->next_end_time - time(NULL),
                                               arec_allocation_callback, aa);
    } else {

        /* if the allocation's end time is closer than the closest in the timer,
           insert the new time event in front of the old time event*/

        if(end_alloc_time <= arec->next_end_time) {

            /* remove current time event for the allocation from timer */
            arec->tim->unset(arec->tim, arec->next_end_tim_id);
      
            /* insert the new time event */
            arec->next_end_time = alloc->end_time(alloc);
            arec->next_end_alloc = alloc;
            aa->alloc = alloc;
            aa->arec = arec;
            arec->next_end_tim_id = arec->tim->set(arec->tim,
                                                   arec->next_end_time - time(NULL),
                                                   arec_allocation_callback, aa);
      
        }
    }

    arec->end_allocs->insert(arec->end_allocs, end_alloc_time, alloc); 
    arec->cur_allocs->insert(arec->cur_allocs, alloc->first_addr(alloc), alloc);

    sync_cache(arec);
}

/*------------------------------------------------------------------*/

static void insert_asa(arec *arec, addr_range *ar){

    int end_asa_time;
    ar_arec *aa;

    end_asa_time = ar->end_time(ar);
  
    aa = (ar_arec *)malloc(sizeof(ar_arec));

    /* insert the asa into list and plist structures */
    arec->asa_ranges->insert(arec->asa_ranges, ar->first_addr(ar), ar);
    arec->asa_end->insert(arec->asa_end, end_asa_time, ar);

    if( arec->asa_ranges->is_empty(arec->asa_ranges)){

   
        arec->next_asa_end_time = ar->end_time(ar);
        aa->ar = ar;
        aa->arec = arec;
        arec->next_end_tim_id = arec->tim->set(arec->tim,
                                               arec->next_asa_end_time - time(NULL),
                                               arec_asa_callback, aa);
    
    }else{
    
        /* is this addr_range's end time closer than the current */
        if( end_asa_time <= arec->next_asa_end_time ){
      
            /* remove current time event for the asa from timer */
            arec->tim->unset(arec->tim, arec->next_asa_end_tim_id);
      
            /* insert the new time event */
            arec->next_asa_end_time = ar->end_time(ar);
            aa->ar = ar;
            aa->arec = arec;
      
            arec->next_asa_end_tim_id = arec->tim->set(arec->tim,
                                                       arec->next_asa_end_time - time(NULL),
                                                       arec_asa_callback, aa);
      
        }
    }
  
    sync_cache(arec);          
}

/*------------------------------------------------------------------*/

static void insert_own(arec *arec, allocation *alloc){

  
    arec->own_allocs->insert(arec->own_allocs, alloc);

    /* insert own allocation to end_allocs and cur_allocs lists */
    arec->insert(arec, alloc);
  
}

/*------------------------------------------------------------------*/
static void insert_claim(arec *arec, addr_range *ar) {
    arec->cur_claims->insert(arec->cur_claims, ar);
}

/*------------------------------------------------------------------*/

static void remove_alloc(arec *arec, allocation *alloc){

    int pos;

    if( alloc == arec->next_end_alloc){
        allocation *next_alloc;
        int time_val;
        alloc_arec *aa = malloc(sizeof(alloc_arec));
    
        arec->tim->unset(arec->tim, arec->next_end_tim_id);

        arec->end_allocs->remove(arec->end_allocs, 0);

    
        next_alloc = arec->end_allocs->first(arec->end_allocs);

        if( next_alloc != NULL ){
      
            aa->alloc  = next_alloc;
            aa->arec = arec;
      
            arec->next_end_alloc = next_alloc;
      
            time_val = next_alloc->end_time(next_alloc) - time(NULL);	   
      
            arec->next_end_tim_id =  arec->tim->set(arec->tim, time_val, arec_allocation_callback, aa);
      
            arec->next_end_time = next_alloc->end_time(next_alloc);
        }
    
    }
  
    /* remove from own_allocs list if exists */
    pos = arec->own_allocs->find(arec->own_allocs, alloc);

    if(pos != -1 )
        arec->own_allocs->remove(arec->own_allocs, pos);

    /* remove from cur_allocs list */
    pos = arec->cur_allocs->find(arec->cur_allocs, alloc);

    if(pos != -1 )
        arec->cur_allocs->remove(arec->cur_allocs, pos);

    /* remove from end_allocs list */

    pos = arec->end_allocs->find(arec->end_allocs, alloc);

    if(pos != -1 )
        arec->end_allocs->remove(arec->end_allocs, pos);
  

}
/*------------------------------------------------------------------*/
void remove_claim(arec *arec, addr_range *ar) {
    int pos;

    pos = arec->cur_claims->find(arec->cur_claims, ar);
    if(pos != -1) {
        arec->cur_claims->elem_at(arec->cur_claims, pos);
    }
}
/*------------------------------------------------------------------*/
/*
 * The overlap function returns a list of all address range overlaps
 *
 */
static list *overlap(arec *arec, addr_range *check){
    list *l = list_new();
    int i;
    allocation *alloc;
    addr_range *ar;    

    for(i = 0; i < arec->cur_allocs->length(arec->cur_allocs); i++) {
    
        alloc = arec->cur_allocs->elem_at(arec->cur_allocs, i);
        ar = alloc->range(alloc);

        if(ar->overlap(ar, check)) {
            l->insert(l, alloc);
        }
    }
    return l;  
}

/* Find out how many addresses are allocated in a specific address range. */
static int in_use(arec *arec, addr_range *check){
    int count = 0;
    int i = 0;
    unsigned long check_first, check_last, alloc_first, alloc_last;
    allocation *alloc;
    check_first = ntohl(check->first_addr(check));
    check_last = ntohl(check->last_addr(check));
    for(i = 0;i < arec->cur_allocs->length(arec->cur_allocs);i++){
        alloc = arec->cur_allocs->elem_at(arec->cur_allocs, i);
        alloc_first = ntohl(alloc->first_addr(alloc));
        alloc_last = ntohl(alloc->last_addr(alloc));
        if(!(alloc->range(alloc)->overlap(alloc->range(alloc), check))){
            /* Ranges not overlapping. */
            continue;
        }
        if(alloc_first > check_first){
            if(alloc_last < check_last){
                count = count + (alloc_last - alloc_first + 1);
            } else {
                count = count + (check_last - alloc_first + 1);
            }
        } else {
            if(alloc_last > check_last){
                count = count + (check_last - check_first + 1);
            } else {
                count = count + (alloc_last - check_first + 1);
            }
        }
    }
    return count;
}

/*------------------------------------------------------------------*/
static list *overlap_claims(arec *arec, addr_range *check){
    list *l = list_new();
    int i;
    addr_range *ar;    

    for(i = 0; i < arec->cur_claims->length(arec->cur_claims); i++) {
    
        ar = arec->cur_claims->elem_at(arec->cur_claims, i);

        if(ar->overlap(ar, check)) {
            l->insert(l, ar);
        }
    }
    return l;  
}

/*------------------------------------------------------------------*/
/*
 * The sel_addr function implements the "selecting an address" algorithm
 * described in section 3.2.7 of draft-ietf-malloc-aap-03.txt. It first
 * tries to find an unused (i.e., not allocated or preallocated) range
 * of addresses, and if that fails it will pick the newest preallocated
 * range.
 *
 * The basic steps of the implementation are:
 *
 *  o Insert "guards" in the list of allocations,
 *  o scan the list of allocations for holes,
 *    as well as finding a suitable preallocation
 *    to use if to holes are found.
 *
 * The "guards" are used to mark the parts of the address space that
 * cannot be allocated, i.e., the inverse of the set of ASA message
 * ranges received from the prefix coordinator.
 *
 * The address space can be visualized as:
 *  ____________________________________________________________
 *  |                  ! # ##  !                               |
 *  |__________________!_#_##__!_______________________________|
 *  0.0.0.0                                      255.255.255.255
 *
 * where the ! marks the start and end of the address range that can
 * be used for allocation by AAP. The # marks addresses that are 
 * already allocated. The "guards" are now inserted:
 *  ____________________________________________________________
 *  |##################! # ##  !###############################|
 *  |##################!_#_##__!###############################|
 *  0.0.0.0                                      255.255.255.255
 *
 * This will have the effect that the area outside of the allocatable
 * range(s) will be considered as being allocated, and will not be scanned
 * for holes.
 *
 */
static addr_range *sel_addr(arec *arec, int num, unsigned int end_time) {

    list *asa_inverse; /* contains the address space that is not covered by asa */
    int i, pos;
    unsigned long first, last, diff;
    addr_range *a_range;
    allocation *alloc;
    allocation *asa_alloc;
    addr_range *asa_addr_range;
    addr_range *ar_result = NULL;
    allocation *newest_pre_alloc=NULL;
  
    asa_inverse = list_new(); 
    first = 0;
 
    /* insert the inverse of the asa into allocs list */
    for(i=0 ; i < arec->asa_ranges->length(arec->asa_ranges) ; i++){

        a_range = arec->asa_ranges->elem_at(arec->asa_ranges,i);
        last = ntohl(a_range->first_addr(a_range)) - 1; /* convert to host byte order */
        diff = last - first;

        /* is there space between */
        if( diff > 1 ){
      
            asa_addr_range = addr_range_new(htonl(first), htonl(last), 0);
            asa_alloc = allocation_new(0, asa_addr_range, 0, ALLOCATION_OWN);
      
            arec->cur_allocs->insert(arec->cur_allocs,first, asa_alloc);
            asa_inverse->insert(asa_inverse, asa_alloc);
      
        }
    
        first = ntohl(a_range->last_addr(a_range)) + 1;
   
    }
  
    /* insert a guarding upper bound (255.255.255.255 - 255.255.255.255) */
    asa_addr_range = addr_range_new(htonl(first), htonl(0xffffffff), 0);
    asa_alloc = allocation_new(0, asa_addr_range, 0, ALLOCATION_OWN);
    arec->cur_allocs->insert(arec->cur_allocs, first, asa_alloc);
    asa_inverse->insert(asa_inverse, asa_alloc);

  
    for(i = 0; i < arec->cur_claims->length(arec->cur_claims); i++) {
        a_range = arec->cur_claims->elem_at(arec->cur_claims, i);
        first = ntohl(a_range->first_addr(a_range)); /* convert to host byte order */
        last = ntohl(a_range->last_addr(a_range)); /* convert to host byte order */

        asa_addr_range = addr_range_new(htonl(first), htonl(last), 0);
        asa_alloc = allocation_new(0, asa_addr_range, 0, ALLOCATION_OWN);
    
        arec->cur_allocs->insert(arec->cur_allocs,first, asa_alloc);
        asa_inverse->insert(asa_inverse, asa_alloc);

    }
  
  
    first = 0;

    /* here comes Adam's find_best(allocs) */

  
    for(i=0 ; i < arec->cur_allocs->length(arec->cur_allocs) ; i++){

        /*    addr_range *ar;*/
    
        alloc = arec->cur_allocs->elem_at(arec->cur_allocs, i);

        /*    ar = alloc->range(alloc);
              ar->debug_print(ar);*/
    
        last = ntohl(alloc->first_addr(alloc)); /* convert to host byte order */

        diff = last - first;

        /* choose the newest preallocated range */
        if( newest_pre_alloc != NULL){
      
            if( alloc->is_prealloc(alloc) &&
                (ntohl(alloc->last_addr(alloc)) - ntohl(alloc->first_addr(alloc))) >= num  &&
                (newest_pre_alloc->start_time(newest_pre_alloc) > alloc->start_time(alloc)) ){
	
                newest_pre_alloc = alloc;
	
            }
        }else{
            if( alloc->is_prealloc(alloc))
                newest_pre_alloc = alloc; 
        }
    
        /* is there enough space between */

        if( diff > num && ((num+first) < (0xffffffff))){      
            ar_result = addr_range_new(htonl(first),htonl(first + num - 1), end_time);
            break;
        }

    
        first = ntohl(alloc->last_addr(alloc)) + 1;
   
    }

    /* remove the asa inverse in allocs list */

    for(i=0 ; i < asa_inverse->length(asa_inverse) ; i++){
    
        /* find position of asa inverse */
        allocation *tmp_alloc = asa_inverse->elem_at(asa_inverse, i);  
        pos = arec->cur_allocs->find(arec->cur_allocs, tmp_alloc);

        /* remove it from alloc list */
        arec->cur_allocs->remove(arec->cur_allocs, pos);

        /* delete the asa inverse allocation */
        tmp_alloc->delete(&tmp_alloc); 
    }

    asa_inverse->delete(asa_inverse);


    /* if no vacant range was found, return the most new preallocated range */
    if( ar_result == NULL && newest_pre_alloc != NULL){

        ar_result = addr_range_new(newest_pre_alloc->first_addr(newest_pre_alloc),
                                   newest_pre_alloc->first_addr(newest_pre_alloc) + num,
                                   end_time); 
    
    }

    return ar_result;
}

/*------------------------------------------------------------------*/

static list *allocations(arec *arec){

    return arec->own_allocs;
}

/*------------------------------------------------------------------*/

arec *arec_new(cfg *c, tim *t, addr_range *scope) {
    list *cache_list, *asa_list, *alloc_list, *own_list;
    allocation *alloc;
    int i;
  
    arec *arec = malloc( sizeof( struct arec_ ));

    arec->cfg = c;
    arec->tim = t;
    arec->cur_allocs = plist_new();
    arec->cur_claims = list_new();
    arec->own_allocs = list_new();
    arec->end_allocs = plist_new();
    arec->asa_ranges = plist_new();
    arec->asa_end    = plist_new();
    arec->ana_reqs   = list_new();
  
    arec->no_asas = 1;

    arec->insert_ana = insert_ana;
    arec->insert = insert;
    arec->insert_asa = insert_asa;
    arec->insert_own = insert_own;
    arec->insert_claim = insert_claim;
    arec->remove = remove_alloc;
    arec->remove_claim = remove_claim;
    arec->overlap = overlap;
    arec->in_use = in_use;
    arec->overlap_claims = overlap_claims;
    arec->sel_addr = sel_addr;
    arec->allocations = allocations;
    arec->is_own = is_own;

    /* insert the fake ASA to define the scope we are in
       (if we are not within a large scope; scope is NULL in that
       case). */

    /* read the cached data */

    if(access(c->arec.cache_fname, R_OK) == 0) {
        cache_list = cfg_parse(c->arec.cache_fname);
        asa_list = (list *)cache_list->elem_at(cache_list, 0);
        alloc_list = (list *)cache_list->elem_at(cache_list, 1);
        own_list = (list *)cache_list->elem_at(cache_list, 2);
    
        for(i = 0; i < asa_list->length(asa_list); i++) {
            addr_range *ar = (addr_range *)asa_list->elem_at(asa_list, i);      
            /* check so we don't define a scope twice */
            if(scope != NULL) {
                if(!ar->overlap(ar, scope)) {
                    if(ar->end_time(ar) > time(NULL)) {
                        insert_asa(arec, ar);
                    }
                }
            }
        }
    
        for(i = 0; i < alloc_list->length(alloc_list); i++) {
            alloc = alloc_list->elem_at(alloc_list, i);
            if(alloc->end_time(alloc) > time(NULL)) {
                insert(arec, alloc);
            }
        }

        for(i = 0; i < own_list->length(own_list); i++) {
            alloc = own_list->elem_at(own_list, i);
            if(alloc->end_time(alloc) > time(NULL)) {
                insert_own(arec, alloc);
            }
        }
    }

    if(scope != CFG_LARGE_SCOPE) {
        insert_asa(arec, scope);
    }
  

    return arec;
}

/*------------------------------------------------------------------*/

static int arec_allocation_callback(void *obj, unsigned int n){

    alloc_arec *aa; 
    allocation *alloc;
    arec *arec;

    int pos;
    int time_val;
  
    aa = (alloc_arec *)obj;
    alloc = aa->alloc;
    arec = aa->arec; 


    
    /**
     * remove associated allocation from allocation record
     */
    /*  {
        time_t ltime = time(NULL);
        alloc->debug_print(alloc);
        }*/
    /* remove allocation from cur_allocs */

    if( (pos = arec->cur_allocs->find(arec->cur_allocs, alloc)) != -1 ){
        arec->cur_allocs->remove(arec->cur_allocs, pos);
      
    }else{
    
        printf("arec_allocation_callback: cur_allocs did not contain the allocation\n");
        fflush(NULL);
        abort();
    
    }

  
    /* remove allocation from own_allocs, if it exists there */
    if( (pos = arec->own_allocs->find(arec->own_allocs, alloc)) != -1 ){
        arec->own_allocs->remove(arec->own_allocs, pos);      
    }
  
    /* remove allocations from end_allocs */
    if((pos = arec->end_allocs->find(arec->end_allocs, alloc)) != -1 ){
        arec->end_allocs->remove(arec->end_allocs, pos);
    }


    alloc->delete(&alloc);
    free(aa);
    /* insert next allocation's end time event to timer */
    if( !arec->end_allocs->is_empty(arec->end_allocs) ){
        alloc = arec->end_allocs->first(arec->end_allocs);
        aa = malloc(sizeof(alloc_arec));
        aa->alloc = alloc;
        aa->arec = arec;
        time_val = alloc->end_time(alloc) - time(NULL);
        arec->next_end_tim_id = arec->tim->set(arec->tim,
                                               time_val,
                                               arec_allocation_callback,
                                               aa);
        arec->next_end_alloc = aa->alloc;
        arec->next_end_time = time_val;
    }
    sync_cache(arec);
    return 1;

}

/*------------------------------------------------------------------*/

static int arec_asa_callback(void *obj, unsigned int n){

    ar_arec *aa; 
    addr_range *ar;
    arec *arec;

    int pos;
    int time_val;
  
    aa = (ar_arec *)obj;
    ar = aa->ar;
    arec = aa->arec;
  
    /**
     * remove associated ASA from allocation record
     */

    ar->debug_print(ar);
  
    if( (pos = arec->asa_ranges->find(arec->asa_ranges, ar)) != -1 ){
        arec->asa_ranges->remove(arec->asa_ranges, pos);
    }else{
        perror("arec_asa_callback: asa_ranges did not contained the allocation");
        exit(-17);
    }
    
    /* remove allocations from asa_end */
    arec->asa_end->remove(arec->asa_end,0);
  
    /* insert next allocation's end time event to timer */
    /* if no more asa in the list, do something */
    if( arec->asa_end->is_empty(arec->asa_end) ){
        arec->no_asas = 1;

    }else{
        ar = (addr_range *)(arec->asa_end->first(arec->asa_end));
        aa = malloc(sizeof(ar_arec));
        aa->ar = ar;
        aa->arec = arec;
        time_val = ar->end_time(ar) - time(NULL);
        arec->next_asa_end_tim_id = arec->tim->set(arec->tim,
                                                   time_val,
                                                   arec_asa_callback,
                                                   aa);
        arec->next_asa_end_time = time_val;
    }
    sync_cache(arec);
    return 1;

}

/*------------------------------------------------------------------*/


static int is_own(arec *arec, addr_range *ar){
  
    int len, i;
    addr_range *own_ar;
    allocation *alloc;
    len = arec->own_allocs->length(arec->own_allocs);
  
    for(i=0 ; i < len ; i++){

        alloc = arec->own_allocs->elem_at(arec->own_allocs, i);
        own_ar = alloc->range(alloc);

        /* check if the addr_range overlaps an allocated */
        if( own_ar->overlap(own_ar, ar) == 1){
            return 1;
        }
    }

    return 0;
}

/*------------------------------------------------------------------*/
/*
  set the timer when the last ASA message expires  
*/

/*void set_asa_exp_timer(arec *arec, int time){

  arec->tim->set(arec->tim, time, asa_exp_timer_callback, arec);
  
  }*/ /* Implemented in aap_asa.c */

/*------------------------------------------------------------------*/
/*
  This function is called when the ASA
  expires in the allocation record.  
 */

/*static int asa_exp_timer_callback(void *obj, unsigned int n){*/

/* start to defend the Prefix Coordinator */


/*    return 1;
      
      }*/ /* Function implemented in aap_asa.c. */
