DIGIT                   [0-9]
CHAR                    [a-zA-Z_]

%{
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "cfg.h"
#include "list.h"
#include "addr_range.h"
#include "allocation.h"
#include "ana_req.h"
#include "cfgparser.h"

static void count();

%}

%x comment

%%

"STARTUP_WAIT"          { count(); return(AAP_STARTUP_WAIT); }
"ANNOUNCE_WAIT"         { count(); return(AAP_ANNOUNCE_WAIT); }
"RESEND_WAIT"           { count(); return(AAP_RESEND_WAIT); }
"REPEAT_INTERVAL"       { count(); return(AAP_REPEAT_INTERVAL); }
"ASA_INTERVAL"          { count(); return(AAP_ASA_INTERVAL); }
"ASRP_INTERVAL"         { count(); return(AAP_ASRP_INTERVAL); }


"allocation-record"     { count(); return(ALLOCATION_RECORD); }

"maas"                  { count(); return(MAAS); }

"pc"                    { count(); return(PC); }

"asa"                   { count(); return(ASA); }

"allocation"            { count(); return(ALLOCATION); }

"own"                   { count(); return(OWN); }

"ana"                   { count(); return(ANA); }

"address"               { count(); return(ADDRESS); }

"port"                  { count(); return(PORT); }

"cache"                 { count(); return(CACHE); }

"scope"                 { count(); return(SCOPE); }

"large"                 { count(); return(LARGE); }

"{"                     { count(); return(LBRACE); }

"}"                     { count(); return(RBRACE); }

";"                     { count(); return(SEMICOLON); }

"("                     { count(); return(LPAREN); }

")"                     { count(); return(RPAREN); }

","                     { count(); return(COMMA); }

"="                     { count(); return(ASSIGN); }

{DIGIT}*\.{DIGIT}*\.{DIGIT}*\.{DIGIT}* { count(); yylval.addr = inet_addr(yytext); return(IPADDR); }

{DIGIT}*                { count(); yylval.num = strtoul(yytext, NULL, 10); return(CONSTANT); }

\"([^"])*\"             { count(); yylval.str = strdup(yytext); return(STRING); }

[ \t\v\n\f]             { count(); }

"#".*$                  { count(); /* comments */ }

"//".*$                  { count(); /* comments */ }

"/*"                    { count(); } BEGIN(comment);

.                       { /* ignore bad characters */ }

<comment>[^*\n]*        { count(); } /* eat anything that's not a '*' */
<comment>"*"+[^*/\n]*   { count(); }/* eat up '*'s not followed by '/'s */
<comment>\n             { count(); }
<comment>"*"+"/"        { count(); } BEGIN(INITIAL);

%%

int yywrap() {
   return(1);
}

int column = 0;

static void count() {
   int i;
   for(i = 0; yytext[i] != '\0'; i++)
      if(yytext[i] == '\n') {
         column = 0;
      } else if(yytext[i] == '\t') {
         column += 8 - (column % 8);
      } else {
         column++;
   }

/*   ECHO;*/
}
