/*
 * This is the maasd program, a part of an implementation of the MALLOC
 * architecture.
 *
 * This file contains constructor and methods for an addr_rec object. 
 * The address record contains all the addresses allocated and all client 
 * allocations made for these addresses.
 *
 * Copyright (C) 2001 Hkan Bystrm,Maria Isberg, Pia Lindstm,
 * Fredrik Pettersson, Linus Svensson and ke stmark.
 *
 * The project is supported by Telia Research AB, Lule, Sweden.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 1307 USA
 *
 * The authors of this program can be reached on malloc@deimos.campus.luth.se,
 * or via the project homepage at http://deimos.campus.luth.se/malloc/.
 * The Telia representative is Thomas Eriksson (Thomas.A.Eriksson@telia.se)
 */

#ifndef _ADDR_REC_H_
#define _ADDR_REC_H_

#include "addr_range.h"
#include "debug.h"
#include "globals.h"
#include "madcap_alloc.h"
#include "madcap_cfg.h"
#include "madcap_error.h"
#include "madcap_plist.h"
#include "tim.h"
#include "list.h"

typedef struct addr_rec_ addr_rec;

enum addr_rec_constants_ {
    /* Requested lifetime when allocating addr = 60 days */
    ADDR_REC_REQ_TIME = 86400*60,
    /* Min remaining lifetime of an address = 3 days */
    ADDR_REC_MIN_TIME = 86400*3    
};

/* The addr_rec object. */
struct addr_rec_ {

    /* The configuration file */
    madcap_cfg *cfg;

    /* The timer for the addr_rec */
    tim *tim;

    /* The list containing the aap:s */
    madcap_list *aap_list;

    /* The list containing the allocations */
    madcap_plist *alloc_list;

    /*
     * Function pointers to public functions of a addr_rec object.
     */
    int (* alloc_addr) (addr_rec *object, 
                        madcap_alloc_flag flag,
                        uint8 *lease_id,
                        uint16 lease_id_len,
                        uint32 scope_id,
                        uint32 req_lease_time,
                        uint32 req_min_lease_time,
                        uint32 req_start_time,
                        uint32 req_max_start_time,
                        uint16 req_no_addr,
                        uint16 req_min_no_addr,
                        madcap_plist *req_addr_ranges,
                        uint32 *alloc_lease_time,
                        uint32 *alloc_start_time,
                        madcap_plist *alloc_addr_ranges);

    int (* renew_addr) (addr_rec *object, 
                        uint8 *lease_id,
                        uint16 lease_id_len,
                        uint32 req_lease_time,
                        uint32 req_min_lease_time,
                        uint32 req_start_time,
                        uint32 req_max_start_time,
                        uint32 *alloc_scope_id,
                        uint32 *alloc_lease_time,
                        uint32 *alloc_start_time,
                        madcap_plist *alloc_addr_ranges);
    
    int (* confirm_prealloc) (addr_rec *object, 
                              uint8 *lease_identifier, 
                              uint16 lease_identifier_length,
                              madcap_plist *conf_pre);
    
    int (* release_prealloc) (addr_rec *object, 
                              uint8 *lease_identifier,
                              uint16 lease_identifier_length);
    
    int (* release_alloc) (addr_rec *object, 
                           uint8 *lease_identifier, 
                           uint16 lease_identifier_length);

    madcap_list *(* get_aap_list)(addr_rec *object);
};

addr_rec *addr_rec_new(madcap_cfg *cfg, tim *tim, madcap_list *aap_list,
                       madcap_list *cache_list);

#endif
