/* This file defines a useful debugging routine, and it is not a necessary
 * part of the MAAS design.
 *
 * The debugging routines allows the user/programmer to turn on selected
 * debugging messages, controllable from the command line arguments passed to
 * MAAS (-v).
 *
 * The pre-defined debugging flags are:
 *
 *      '+' -- turn on all debug messages
 *      'g' -- generic message (madcap_msg)
 *      'a' -- allocation message (addr_rec)
 *      'f' -- format outgoing options (madcap_msg_format_options)
 *      'm' -- madcap messages (madcap_msg_getinfo, -request, -release,
 *             -ack, -nak, -discover, -renew, -offer and -unknown)
 *      'e' -- error messages
 *      's' -- all bytes sent and received
 *      'n' -- networking information (net.c and ?)
 *      'i' -- initiation
 *      'd' -- aapd messages (aap_asa)
 *      '' -- 
 *      '' -- 
 *      '' -- 
 *      '' -- 
 *      '' -- 
 *
 * Please, feel free to add more flags to the above, which is the only place
 * needed to store changes. The flags have to be unique and a single character.
 *
 * Copyright (C) 2001 Hkan Bystrm,Maria Isberg, Pia Lindstm,
 * Fredrik Pettersson, Linus Svensson and ke stmark.
 *
 * The project is supported by Telia Research AB, Lule, Sweden.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 1307 USA
 *
 * The authors of this program can be reached on malloc@deimos.campus.luth.se,
 * or via the project homepage at http://deimos.campus.luth.se/malloc/.
 * The Telia representative is Thomas Eriksson (Thomas.A.Eriksson@telia.se)
 */


#ifndef _DEBUG_H_
#define _DEBUG_H_

#include <stdio.h>
#include <stdarg.h>
#include "madcap_error.h"

/**
 * Enable printing debug messaged and initialize so that only DEBUG messages
 * with a flag in flag_list will be printed. If the flag is "+", we enable all
 * DEBUG messages.
 *
 * @param flags A string of characters for whose DEBUG messages are
 * to be enabled.
 */
extern void DEBUG_INIT(char* flags);

/**
 * Check if debug flag is enabled.
 *
 * @param flag The flag to check.
 * @return If flag not set FAILURE(-1) is returned, else SUCCESS(0) is returned.
 */
extern int DEBUG_ENABLED(char flag);

/**
 * Print debug message if flag is enabled.
 * Works as printf, but with an extra first argument.
 *
 * @param flag The flag that has to be set for this debug message to be printed.
 */
extern void DEBUG (char flag, char* format, ...);


#endif
