/*
 * This is the maasd program, a part of an implementation of the MALLOC
 * architecture.
 *
 * This file contains...
 *
 * Copyright (C) 2001 Hkan Bystrm,Maria Isberg, Pia Lindstm,
 * Fredrik Pettersson, Linus Svensson and ke stmark.
 *
 * The project is supported by Telia Research AB, Lule, Sweden.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 1307 USA
 *
 * The authors of this program can be reached on malloc@deimos.campus.luth.se,
 * or via the project homepage at http://deimos.campus.luth.se/malloc/.
 * The Telia representative is Thomas Eriksson (Thomas.A.Eriksson@telia.se)
 */

#ifndef _GLOBALS_H_
#define _GLOBALS_H_

#include "madcap_list.h"
#include "aap.h"

/*
 * Definition of certain important data types.
 * The definitions should be chosen according to datatypes existing on the
 * present platform.
 */
typedef char           int8; 
typedef unsigned char  uint8;
typedef short          int16;
typedef unsigned short uint16;
typedef long           int32;
typedef unsigned long  uint32;

typedef struct scope_ scope;
typedef struct aap_list_item_ aap_list_item;

typedef struct address_range{
    uint32 start;
    uint32 end;
} address_range;

typedef struct name_set_node{
    char *language_tag;
    char *name;
} name_set_node;

struct scope_ {
    /* The address range containing the set of addresses */
    address_range addresses;
    /* The list containing character-string names indexed
       by the language tag */
    madcap_list *name_set;
    /* The default TTL for the scope */
    unsigned int ttl;
    /* Given in % of current number of addresses allocated */
    unsigned int min_free_addr;
    /* Number of addresses madcap shall allocate when server
       is started */
    unsigned int start_no_addr;
};

/*typedef struct aap_ aap;*/

struct aap_list_item_ {
    aap *aap;
    uint32 scope_id;
};

enum bool {
    FALSE = 0,
    TRUE = 1   /* TRUE should not be treated as 1, but rather as not zero. */
};

/**
 * Use this instead of exit(EXIT_FAILURE).
 * Gives information about which file and line the abortion of the program
 * occurred.
 *
 * @param s The abort message.
 */
#define ABORT(s) {fprintf(stderr, "Execution aborted at %s[%d]: %s\n", \
                          __FILE__, __LINE__, s); \
                  exit(EXIT_FAILURE); \
                 }

/**
 * Used to test conditions.
 */
#define ASSERT(c, s)                                                           \
    {if (!(c)) {                                                               \
        fprintf(stderr, "Assertion failed at %s[%d]. %s\n",                    \
                __FILE__, __LINE__, s);                                        \
        exit(EXIT_FAILURE);                                                    \
    }}

#endif /* _GLOBALS_H_ */
