/*
 * This is the maasd program, a part of an implementation of the MALLOC
 * architecture.
 *
 * This file contains constructor for a madcap_alloc object. 
 *
 * Copyright (C) 2001 Hkan Bystrm,Maria Isberg, Pia Lindstm,
 * Fredrik Pettersson, Linus Svensson and ke stmark.
 *
 * The project is supported by Telia Research AB, Lule, Sweden.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 1307 USA
 *
 * The authors of this program can be reached on malloc@deimos.campus.luth.se,
 * or via the project homepage at http://deimos.campus.luth.se/malloc/.
 * The Telia representative is Thomas Eriksson (Thomas.A.Eriksson@telia.se)
 */
#ifndef _MADCAP_ALLOC_H_
#define _MADCAP_ALLOC_H_

#include "debug.h"
#include "madcap_plist.h"
#include "globals.h"
#include "madcap_error.h"


typedef enum madcap_alloc_flag_ {
    MADCAP_ALLOC_ALLOC,
    MADCAP_ALLOC_PREALLOC,
    MADCAP_ALLOC_TEMP
} madcap_alloc_flag;

typedef struct madcap_client_alloc_ madcap_client_alloc;
typedef struct madcap_alloc_ madcap_alloc;

/* The madcap_client_alloc object */
struct madcap_client_alloc_ {
    uint8 *lease_id;
    uint16 lease_id_length;
    madcap_alloc_flag flag;
    uint32 start_time;
    uint32 end_time;
    madcap_alloc *owner;
    tim_id_t tim_id;
};

/* The madcap_alloc object */
struct madcap_alloc_ {

    /* The address */
    uint32 address;
    /* The scope of the address */
    uint32 scope_id;
    /* The end time of the address */
    uint32 address_end_time;
    /* The list containing the allocations */
    madcap_plist *client_allocs_list;

};

/**
 * The constructor of a madcap_alloc object.This is a representation
 * for a multicast address allocated by madcap.
 * @param address The address allocated
 * @param scope_id The scope of the address
 * @param address_end_time The end time of the address
 * @return A pointer to a madcap_msg object.
 */
madcap_alloc *madcap_alloc_new(uint32 address, uint32 scope_id,
                               uint32 address_end_time);

#endif
