/*
 * This is the maasd program, a part of an implementation of the MALLOC
 * architecture.
 *
 * This file contains the different configuration settings for madcap.
 *
 * Copyright (C) 2001 Hkan Bystrm,Maria Isberg, Pia Lindstm,
 * Fredrik Pettersson, Linus Svensson and ke stmark.
 *
 * The project is supported by Telia Research AB, Lule, Sweden.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 1307 USA
 *
 * The authors of this program can be reached on malloc@deimos.campus.luth.se,
 * or via the project homepage at http://deimos.campus.luth.se/malloc/.
 * The Telia representative is Thomas Eriksson (Thomas.A.Eriksson@telia.se)
 */
#ifndef _MADCAP_CFG_H_
#define _MADCAP_CFG_H_

#include "globals.h"

/* The type of configuration */
typedef enum {
    CFG_MADCAP
} mc_cfg_type;

typedef struct madcap_cfg_ madcap_cfg;

/* The madcap_cfg object */
struct madcap_cfg_ {

    /* The name of the configuration */
    char *name;

    /* The type of the configuration */
    mc_cfg_type type;

    /* File name */
    char *filename;    

    /* Array of scopes */
    /* FIXME: set how many scopes the list should be able to hold */
    scope scopes[20];

    /* Number of scopes */
    int scope_count;

    /* Port */
    int port;

    /* Timers in sec. */
    int CSA;
    int DELAY, ALLOC_TIME, RESPONSE_DELAY, OFFER_HOLD, RCI, XRI;

    /* File to store address record. */
    int (* get_addr_rec_file) (madcap_cfg *object, char *filename);

    /* An array of defined scopes. */
    int (* get_scopes) (madcap_cfg *object, scope *scps[]);

    /* Port of selected scope. */
    int (* get_port) (madcap_cfg *object, char *name, int *port);

    /* Number of scopes */
    int (* get_number_of_scopes)(madcap_cfg *object, int *count);

    /* Timers. [sec] */
    int (* get_clock_skew_allowance) (madcap_cfg *object, char *name,
                                      uint32 *csa);
    int (* get_discover_delay) (madcap_cfg *object, char *name, int *delay);
    int (* get_extra_allocation_time) (madcap_cfg *object, char *name,
                                       int *alloc_time);
    int (* get_no_response_delay) (madcap_cfg *object, char *name,
                                   int *response_delay);
    int (* get_offer_hold) (madcap_cfg *object, char *name, int *offer_hold);
    int (* get_response_cache_interval) (madcap_cfg *object, char *name,
                                         int *rci);
    int (* get_xid_reuse_interval) (madcap_cfg *object, char *name, int *xri);
    
};

/**
 * The mc_cfg_parse function calls the mcparse function (generated by the
 * madcap_cfgparser.y file).
 *
 * @param filename The file to read the configuration from.
 * @return A pointer to a list of configure objects.
 */
madcap_list *mc_cfg_parse(char *filename);

/**
 * The constructor of a madcap_cfg object.
 *
 * @return A madcap_cfg-object on success, a NULL-pointer in case
 * of failure.
 */
madcap_cfg *madcap_cfg_new();

#endif
