%{
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "madcap_cfg.h"
#include "madcap_list.h"
#include "madcap_plist.h"
#include "globals.h"
#include "madcap_alloc.h"

extern char yytext[];
extern int col;

madcap_list *mc_cfgparser_list = NULL;

static char *strtoname(char *str); 
int mcerror(char *s);

int i;
  
%}

%union {
    unsigned int num;
    unsigned long addr;
    char *str;
    uint32 num32;
    madcap_list *l;
    madcap_plist *pl;
    address_range ar;
    name_set_node *nsn;
    madcap_cfg *cfg;
    scope *scope;
    madcap_alloc *mc_alloc;
    madcap_client_alloc *m_c_a;
    uint8 *lid;
    struct constants {
        int CSA, DELAY, ALLOC_TIME, RESPONSE_DELAY, OFFER_HOLD, RCI, XRI;
    } constants;
   
}

%token MC_CSA, MC_DELAY, MC_ALLOC_TIME, MC_RESPONSE_DELAY, MC_OFFER_HOLD
%token MC_RCI, MC_XRI

%token PORT SCOPE AR NS CACHEFILENAME

%token TTL MFA SNA

%token ALLOC ADDR_REC CALLOC

%token LBRACE RBRACE SEMICOLON COLON

%token START END

%token LPAREN RPAREN COMMA
%token ASSIGN

%token <num> CONSTANT
%token <addr> IPADDR
%token <str> STRING

%token MADCAP

%type <num> csa, delay, alloc_time, response_delay, offer_hold, rci, xri

%type <num32> start, end

%type <constants> mc_constants
%type <num> port ttl mfa sna

%type <cfg> mc_cfg config_def

%type <l> config_list scope_list cache alloc_list lid_list name_set_list name_set
%type <pl> calloc_list

%type <scope> scope

%type <mc_alloc> alloc
%type <lid> lease_id
%type <m_c_a> calloc

%type <ar> address_range
%type <nsn> name_set_node
%type <str> cachefilename

%%

config
: config_list { mc_cfgparser_list = $1; }
| cache { mc_cfgparser_list = $1; }
;

config_list
      : config_def { $$ = madcap_list_new(); $$->insert($$, $1); }
      | config_list config_def { $$ = $1; $$->insert($$, $2); }
      ;

cache
: ADDR_REC LBRACE alloc_list RBRACE SEMICOLON { $$ = $3; }
;

alloc_list
: { $$ = madcap_list_new(); }
| alloc { $$ = madcap_list_new(); $$->insert($$, $1); }
| alloc_list alloc { $$ = $1; $$->insert($$, $2); }
;

alloc
: ALLOC LPAREN CONSTANT COMMA CONSTANT COMMA CONSTANT RPAREN LBRACE
  calloc_list
  RBRACE SEMICOLON {
    $$ = madcap_alloc_new($3, $5, $7);
    $$->client_allocs_list = $10;
    for(i = 0;i < $$->client_allocs_list->length($$->client_allocs_list);i++){
        madcap_client_alloc *tmp_calloc = (madcap_client_alloc *)
            $$->client_allocs_list->elem_at($$->client_allocs_list, i);
        tmp_calloc->owner = $$;
    }
}
;

calloc_list
: { $$ = madcap_plist_new(); }
| calloc { $$ = madcap_plist_new(); $$->insert($$, $1->end_time, $1); }
| calloc_list calloc { $$ = $1; $$->insert($$, $2->end_time, $2); }
;

calloc
: CALLOC LPAREN lease_id COMMA CONSTANT COMMA CONSTANT COMMA CONSTANT COMMA
  CONSTANT RPAREN SEMICOLON {
    $$ = (madcap_client_alloc *)malloc(sizeof(madcap_client_alloc));
    $$->lease_id = $3;
    $$->lease_id_length = $5;
    $$->flag = $7;
    $$->start_time = $9;
    $$->end_time = $11;
}
;

lease_id
: lid_list {
    $$ = (uint8 *)malloc($1->length($1) * sizeof(uint8));
    for(i = 0;i < $1->length($1);i++){
        $$[i] = *(uint8 *)$1->elem_at($1, i);
        free((uint8 *)$1->elem_at($1, i));
    }
    $1->delete($1);
}
;

lid_list
: CONSTANT {
    uint8 *tmp_lidbit = (uint8 *)malloc(sizeof(uint8));
    *tmp_lidbit = $1;
    $$ = madcap_list_new();
    $$->insert($$, tmp_lidbit);
}
| lid_list COLON CONSTANT {
    uint8 *tmp_lidbit = (uint8 *)malloc(sizeof(uint8));
    *tmp_lidbit = $3;
    $$ = $1;
    $$->insert($$, tmp_lidbit);
}
;

config_def
:  MADCAP STRING LBRACE mc_cfg RBRACE SEMICOLON {
    $$ = $4;
    $$->name = strtoname($2);
    $$->type = CFG_MADCAP; }
;

mc_cfg
: scope_list port mc_constants cachefilename
{
    $$ = madcap_cfg_new();
    $$->CSA = $3.CSA;
    $$->DELAY = $3.DELAY;
    $$->ALLOC_TIME = $3.ALLOC_TIME;
    $$->RESPONSE_DELAY = $3.RESPONSE_DELAY;
    $$->OFFER_HOLD = $3.OFFER_HOLD;
    $$->RCI = $3.RCI;
    $$->XRI = $3.XRI;
    $$->port = $2;
    $$->filename = $4;
    for( i = 0; i < $1->length($1); i++){
        $$->scopes[i] = *(scope *)$1->elem_at($1,i);
    }
    $$->scope_count = $1->length($1);
};

mc_constants
: csa delay alloc_time response_delay offer_hold rci xri
{
    $$.CSA = $1;
    $$.DELAY = $2;
    $$.ALLOC_TIME = $3;
    $$.RESPONSE_DELAY = $4;
    $$.OFFER_HOLD = $5;
    $$.RCI = $6;
    $$.XRI = $7;}
;

port
: PORT ASSIGN CONSTANT SEMICOLON { $$ = $3; }
;

cachefilename
: CACHEFILENAME ASSIGN STRING SEMICOLON { $$ = strtoname($3); }

scope_list
: scope {
    $$ = madcap_list_new();
    $$->insert($$, $1);
}
|scope_list scope {
    $$ = $1; $$->insert($$, $2);
} 
;

scope
: SCOPE LBRACE address_range name_set ttl mfa sna RBRACE SEMICOLON
{
    $$ = (scope *)malloc(sizeof(scope));
    $$->addresses = $3;
    $$->name_set = $4;
    $$->ttl = $5;
    $$->min_free_addr = $6;
    $$->start_no_addr = $7;
}
;

csa
: MC_CSA ASSIGN CONSTANT SEMICOLON { $$ = $3; }
;

delay
: MC_DELAY ASSIGN CONSTANT SEMICOLON { $$ = $3; }
;

alloc_time
: MC_ALLOC_TIME ASSIGN CONSTANT SEMICOLON { $$ = $3; }
;

response_delay
: MC_RESPONSE_DELAY ASSIGN CONSTANT SEMICOLON { $$ = $3; }
;

offer_hold
: MC_OFFER_HOLD ASSIGN CONSTANT SEMICOLON { $$ = $3; }
;

rci
: MC_RCI ASSIGN CONSTANT SEMICOLON { $$ = $3; }
;

xri
: MC_XRI ASSIGN CONSTANT SEMICOLON { $$ = $3; }
;

ttl
: TTL ASSIGN CONSTANT SEMICOLON { $$ = $3; }
;

mfa
: MFA ASSIGN CONSTANT SEMICOLON { $$ = $3; }
;

sna
: SNA ASSIGN CONSTANT SEMICOLON { $$ = $3; }
;

address_range
: AR LBRACE start end RBRACE SEMICOLON {
    $$.start = $3;
    $$.end = $4;
}
;

start
: START ASSIGN IPADDR SEMICOLON { $$ = $3; }
;

end
: END ASSIGN IPADDR SEMICOLON { $$ = $3; }
;

name_set
: NS LBRACE name_set_list RBRACE SEMICOLON {
    $$ = $3;
}
;

name_set_list
: name_set_node {
    $$ = madcap_list_new();
    $$->insert($$, $1);
}
| name_set_list name_set_node {
    $$ = $1;
    $$->insert($$, $2);

}
;

name_set_node
: LPAREN STRING COMMA STRING RPAREN SEMICOLON {
    $$ = (name_set_node *)malloc(sizeof(name_set_node));
    $$->language_tag = strtoname($2);
    $$->name = strtoname($4);
}
;

%%


int mcerror(char *s) {
        fflush(stdout);
        printf("\n%*s\n%*s\n", col, "^", col, s);
	return 1;
}

static char *strtoname(char *str) {
  char *s = strdup(str + 1);
  s[strlen(s) - 1] = 0;
  free(str);
  return s;
}
