DIGIT   [0-9]
CHAR    [a-zA-Z_]

%{
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "madcap_cfg.h"
#include "madcap_list.h"
#include "madcap_plist.h"
#include "madcap_alloc.h"
#include "madcap_cfgparser.h"

static void count();

%}

%x comment

%%

"CSA"             { count(); return(MC_CSA); }
"DELAY"           { count(); return(MC_DELAY); }
"ALLOC_TIME"      { count(); return(MC_ALLOC_TIME); }
"RESPONSE_DELAY"  { count(); return(MC_RESPONSE_DELAY); }
"OFFER_HOLD"      { count(); return(MC_OFFER_HOLD); }
"RCI"             { count(); return(MC_RCI); } 
"XRI"             { count(); return(MC_XRI); }


"madcap"     { count(); return(MADCAP); }

"port"                  { count(); return(PORT); }

"cachefilename"         { count(); return(CACHEFILENAME); }

"scope"                 { count(); return(SCOPE); }

"ttl"                   { count(); return(TTL); }

"addr_rec"              { count(); return(ADDR_REC); }

"min_free_addr"         { count(); return(MFA); }

"alloc"                 { count(); return(ALLOC); }

"calloc"                { count(); return(CALLOC); }

"start_no_addr"         { count(); return(SNA); }

"address_range"         { count(); return(AR); }

"name_set"              { count(); return(NS); }

"start"                 { count(); return(START); }

"end"                   { count(); return(END); }

"{"                     { count(); return(LBRACE); }

"}"                     { count(); return(RBRACE); }

";"                     { count(); return(SEMICOLON); }

":"                     { count(); return(COLON); }

"("                     { count(); return(LPAREN); }

")"                     { count(); return(RPAREN); }

","                     { count(); return(COMMA); }

"="                     { count(); return(ASSIGN); }

{DIGIT}*\.{DIGIT}*\.{DIGIT}*\.{DIGIT}* { count(); mclval.addr =  ntohl(inet_addr(yytext)); return(IPADDR); }

{DIGIT}*                { count(); mclval.num = strtoul(yytext, NULL, 10); return(CONSTANT); }

\"([^"])*\"             { count(); mclval.str = strdup(yytext); return(STRING); }

[ \t\v\n\f]             { count(); }


"#".*$                  { count(); /* comments */ }

"//".*$                  { count(); /* comments */ }


"/*"                    { count(); } BEGIN(comment);

.                       { /* ignore bad characters */ }

<comment>[^*\n]*        { count(); } /* eat anything that's not a '*' */
<comment>"*"+[^*/\n]*   { count(); }/* eat up '*'s not followed by '/'s */
<comment>\n             { count(); }
<comment>"*"+"/"        { count(); } BEGIN(INITIAL);

%%

int yywrap() {
   return(1);
}

int col = 0;

static void count() {
   int i;
   for(i = 0; yytext[i] != '\0'; i++)
      if(yytext[i] == '\n') {
         col = 0;
      } else if(yytext[i] == '\t') {
         col += 8 - (col % 8);
      } else {
         col++;
   }

/*   ECHO;*/
}

