/*
 * This is the maasd program, a part of an implementation of the MALLOC
 * architecture.
 *
 * This file contains constructor and methods for a madcap_error object.
 *
 * Copyright (C) 2001 Hkan Bystrm,Maria Isberg, Pia Lindstm,
 * Fredrik Pettersson, Linus Svensson and ke stmark.
 *
 * The project is supported by Telia Research AB, Lule, Sweden.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 1307 USA
 *
 * The authors of this program can be reached on malloc@deimos.campus.luth.se,
 * or via the project homepage at http://deimos.campus.luth.se/malloc/.
 * The Telia representative is Thomas Eriksson (Thomas.A.Eriksson@telia.se)
 */

#ifndef _MADCAP_ERROR_H_
#define _MADCAP_ERROR_H_


/* Standard return values. */
enum return_values {
    FAILURE = -1,
    SUCCESS = 0
};

/* Standard error types. */
/* IMPORTANT!!! When adding new error type, make sure you add an apropriate
   default error message in madcap_error.c */
enum madcap_error_type {
    MC_ERR_FAMILY,     /* Bad address family. */
    MC_ERR_VERSION,    /* Wrong version of MADCAP message. */
    MC_ERR_MSGTYPE,    /* Bad message type. */
    MC_ERR_XID,        /* Unknown transaction ID. */
    MC_ERR_LEASE_ID,   /* Unknown lease ID. */
    MC_ERR_OPTIONSLEN, /* Length of options field too long. */
    MC_ERR_OPTIONS,    /* Bad syntax in options field. */
    MC_ERR_MSGLEN,     /* Length of MADCAP message too short. */
    MC_ERR_SCOPE,      /* Bad scope. */
    MC_ERR_NUMADDR,    /* Bad desired number of addresses (e.g., max < min). */
    MC_ERR_LTIME,      /* Bad desired lifetimes (e.g., max < min). */
    MC_ERR_STIME,      /* Bad start time durations. */
    MC_ERR_TIMECONF,/*Requested start times conflict with requested lifetimes.*/
    MC_ERR_FAILED,     /* The operation failed. */
    MC_ERR_LEASE,  /*The addresses were not the ones that had been allocated.*/
    MC_ERR_NONEST,     /* Nesting state undetermined at this time. */
    MC_ERR_OUTOFADDR,  /* Not enough addresses available. */
    MC_ERR_ADDRLTIME,  /* Not enough lifetime on available addresses. */
    MC_ERR_CREATE_DISCOVER, /* Could not create DISCOVER message. */
    MC_ERR_CREATE_OFFER,    /* Could not create OFFER message. */
    MC_ERR_CREATE_REQUEST,  /* Could not create REQUEST message. */
    MC_ERR_CREATE_RENEW,    /* Could not create RENEW message. */
    MC_ERR_CREATE_ACK,      /* Could not create ACK message. */
    MC_ERR_CREATE_NAK,      /* Could not create NAK message. */
    MC_ERR_CREATE_RELEASE,  /* Could not create RELEASE message. */
    MC_ERR_CREATE_GETINFO,  /* Could not create GETINFO message. */
    MC_ERR_SEND_OFFER,      /* Could not send OFFER message. */
    MC_ERR_SEND_ACK,        /* Could not send ACK message. */
    MC_ERR_SEND_NAK,        /* Could not send NAK message. */
    MC_ERR_UNKNOWN,         /* Type of error unknown. */
    MC_ERR_UNSPECIFIED,     /* The type of error is not specified. */
    MC_ERR_NONEW,     /*Error already collected and no new error has occured.*/
    MC_ERR_CONF,      /*Error in configuration*/
    MC_ERR_MAX /* This should always be the last MC_ERR..., add error above */
};

typedef enum madcap_error_type madcap_error_type;

typedef struct madcap_error madcap_error;

/* Represents a madcap_error object. */
struct madcap_error{

    /* The error number. */
    int mc_errno;

    /* The error message. */
    char *err_message;

    /*
     * Function pointers to public functions of a madcap_error object.
     */
    
    /* Notify which error has ocurred.
       The message is appended to default error message. */
    int (* notify) (madcap_error *object, madcap_error_type err_num,
                    char *message);

    /* Collect the error message as a describing string */
    int (* madcap_errstring_collect) (madcap_error *object, char *msg);

    /* Collect the error message as a number. */
    int (* madcap_errnum_collect) (madcap_error *object,
                                   madcap_error_type *err_num);
    
    /* Translate an error type from number to string. */
    int (* madcap_errstring_translate) (madcap_error *object,
                                        madcap_error_type err_num,
                                        char *error);

    /* Print the description of the error to standard error output.
       The message is appended to the default error message. */
    int (* madcap_perror) (madcap_error *object, const char *message);
};

/**
 * The constructor of a madcap_error object.
 *
 * @return A pointer to a madcap_error-object on succes, a NULL-pointer in
 * case of failure. 
 */
madcap_error *madcap_error_new();

/* The madcap_error object. */
madcap_error *mc_error;
#endif

