/*
 * This is the maasd program, a part of an implementation of the MALLOC
 * architecture.
 *
 * This file contains ...
 *
 * Copyright (C) 2001 Hkan Bystrm,Maria Isberg, Pia Lindstm,
 * Fredrik Pettersson, Linus Svensson and ke stmark.
 *
 * The project is supported by Telia Research AB, Lule, Sweden.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 1307 USA
 *
 * The authors of this program can be reached on malloc@deimos.campus.luth.se,
 * or via the project homepage at http://deimos.campus.luth.se/malloc/.
 * The Telia representative is Thomas Eriksson (Thomas.A.Eriksson@telia.se)
 */


#ifndef _MADCAP_MSG_FORMAT_OPTIONS_H_
#define _MADCAP_MSG_FORMAT_OPTIONS_H_


#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <unistd.h>
#include <string.h>
#include <netdb.h>
#include <arpa/inet.h>
#include "globals.h"
#include "madcap_msg.h"
#include "madcap_plist.h"
#include "debug.h"

typedef struct madcap_msg_format_options_ madcap_msg_format_options;

typedef enum madcap_nak_error_code_ {
    MC_NAK_VALID_REQ_NOT_COMPLETED = 0,
    MC_NAK_INVALID_REQ             = 1,
    MC_NAK_EXCESSIVE_CLK_SKEW      = 2,
    MC_NAK_LEASE_ID_NOT_RECON      = 3,
    MC_NAK_REQ_FEATURE_NOT_SUPPORT = 4,
    MC_NAK_VALID                   = 5
} madcap_nak_error_code;


/* The madcap_msg_nak object. */
struct madcap_msg_format_options_ {

    /* Function pointers. */
    int (* end) (madcap_msg_format_options *object,
                 uint8 **data,
                 int *data_len);
    int (* lease_time) (madcap_msg_format_options *object,
                        uint32 lease_time,
                        uint8 **data,
                        int *data_len);
    int (* server_id) (madcap_msg_format_options *object,
                       uint8 **data,
                       int *data_len);
    int (* lease_id) (madcap_msg_format_options *object,
                      uint8 *lease_id,
                      int lease_id_len,
                      uint8 **data,
                      int *data_len);
    int (* scope) (madcap_msg_format_options *object,
                   uint32 scope_id,
                   uint8 **data,
                   int *len);
    int (* start_time) (madcap_msg_format_options *object,
                        uint32 start_time,
                        uint8 **data,
                        int *data_len);
    int (* scope_list) (madcap_msg_format_options *object,
                        uint8 **data,
                        int *data_len);
    int (* addr_range_list) (madcap_msg_format_options *object,
                             madcap_plist *addr_range_list,
                             uint8 **data,
                             int *data_len);
    int (* current_time) (madcap_msg_format_options *object,
                          uint32 current_time,
                          uint8 **data,
                          int *data_len);
    int (* feature_list) (madcap_msg_format_options *object,
                          uint8 **data,
                          int *data_len);
    int (* retry_time) (madcap_msg_format_options *object,
                        uint32 retry_time,
                        uint8 **data,
                        int *data_len);
    int (* error) (madcap_msg_format_options *object, 
                   madcap_nak_error_code err_code,
                   uint8 *err_data,
                   int err_data_len,
                   uint8 **data,
                   int *data_len);
    int (* get_server_id) (madcap_msg_format_options *object,
                           uint32 *server_id);
    int (* is_server_id) (madcap_msg_format_options *object,
                          uint32 server_id);
    int (* delete) (madcap_msg_format_options *object);
};


/**
 * Creates a new NAK message.
 * 
 * @return The newly created object. The object should be deleted by the delete
 * function. A return value of NULL indicates a failure to create the new
 * message.
 */
madcap_msg_format_options *madcap_msg_format_options_new();


#endif
