#include <stdlib.h>
#include <time.h>

#include "tim.h"

/**
 * tim.c
 *
 * Implements a timer with methods for inserting and extrating
 * time events. Each time event has an associated callback function,
 * which is called when the timer expires. See tim.h for details
 * about the methods.
 *
 * The time events are kept in a priority queue (see plist.h) with the
 * time of the time event used as priority.
 *
 */

struct tim_event {
    int sequence_number;
    unsigned int time;
    tim_callback callback;
    void *obj;
};

/*------------------------------------------------------------------*/

static tim_id_t set(tim *tim, int timeout, tim_callback callback, void *obj) {
    struct tim_event *event;

    event = malloc(sizeof(struct tim_event));

    event->sequence_number = tim->sequence_number;
    tim->sequence_number++;
  
    event->time     = timeout + time(NULL);
    event->callback = callback;
    event->obj      = obj;
    tim->events->insert(tim->events, event->time, event);
    return event->sequence_number; 
}

/*------------------------------------------------------------------*/

struct match_element {
    int sequence_number;
};

static int matching(void *obj, void *element) {
    struct tim_event *e = element;
    struct match_element *me = obj;
    return e->sequence_number == me->sequence_number;
}

static int unset(tim *tim, tim_id_t id) {
    struct match_element me;
    me.sequence_number = id;
    tim->events->remove_element(tim->events, matching, &me);
    return 0; /* XXX */
}

/*------------------------------------------------------------------*/

static int get_next_time(tim *tim, struct timeval *t) {
    t->tv_sec = tim->events->priority(tim->events, 0) - time(NULL);
    t->tv_usec = 0;
    return 1;
}

/*------------------------------------------------------------------*/

static int call_next(tim *tim) {
    struct tim_event *event;
  
    if(tim->events->is_empty(tim->events)) {
        return -1;
    } 
    event = (struct tim_event *)tim->events->first(tim->events);
    tim->events->remove(tim->events, 0);
    event->callback(event->obj, time(NULL));
    free(event);
 
    return 0;
}

/*static int unset_next(tim *tim) {
  return tim->events->remove(tim->events);
}*/

/*------------------------------------------------------------------*/

static int is_empty(tim *tim) {
    return tim->events->is_empty(tim->events);
}

/*------------------------------------------------------------------*/

tim *tim_new() {
    tim *timer;

    timer = malloc(sizeof(tim));
    timer->events        = plist_new();

    timer->sequence_number = 0;
  
    timer->set           = set;
    timer->unset         = unset;
    timer->get_next_time = get_next_time;
    timer->call_next     = call_next;
    /*  timer->unset_next    = unset_next;*/
    timer->is_empty      = is_empty;
    return timer;
}

/*------------------------------------------------------------------*/
